# coding: utf-8
from tkinter import *
from tkinter import messagebox
from tkinter import filedialog
import string
MAXBLOCK = 65536
SAFEBLOCK = 1024*1024
#■　2次元配列の宣言
def array2(N1,N2):return [[0 for j in range(N2)]for k in range(N1)]
#■　リトル・エンディアンの処理
def little16(c): # 2.7ではstringだったがバイナリ・ファイルの入力はバイトのリストになった
    return c[0] + (c[1]<<8)
def little32(c): return c[0] | (c[1]<<8) | (c[2]<<16) | (c[3]<<24)
#■　ファイル書き込み用整数の生成(16ビット用と32ビット用)
def bAr16(X): return X.to_bytes(2, byteorder='little',signed=False)
def bAr32(X): return X.to_bytes(4, byteorder='little',signed=False)
#■　メモリ・エラーを起こさない読み込み
def safe_read(fp, size):
    if size <= 0: return ""
    if size <= SAFEBLOCK: #SAFEBLOCKより小さければ一度に入力
        return fp.read(size)
    data = []
    while size > 0: #SAFEBLOCKより大きいとき分割入力
        block = fp.read(min(size, SAFEBLOCK))
        if not block: break
        for i in range(len(block)): # 2.7ではstringとして扱っていたので
            data.append(block[i])   # data.append(block)だけで良かった。
        size = size - len(block)
    return data
#■ byte表現をPython色表現に変換
HexaTab=('0','1','2','3','4','5','6','7','8','9','A', 'B', 'C', 'D', 'E', 'F')
def hexa2(A):
    B=int(A) & 0xFF; return HexaTab[int(B/16)]+HexaTab[B % 16]
def RGB(R,G,B): return "#%02X%02X%02X" % (R,G,B)
#    return "#" + hexa2(R)+ hexa2(G)+ hexa2(B)
#■ Python色表現からR・G・B要素に分解
HexaValue=[ord('0'), ord('9'), ord('A'), ord('F'),ord('a'),ord('f')]
def HValue(X):    
    D=ord(X)
    if   D>=HexaValue[0] and D<=HexaValue[1]:return D-HexaValue[0]
    elif D>=HexaValue[2] and D<=HexaValue[3]:return D-HexaValue[2]+10
    elif D>=HexaValue[4] and D<=HexaValue[5]:return D-HexaValue[4]+10
    else: return 0
def RED(X)  :return (HValue(X[1])*16+HValue(X[2])) & 0xFF 
def GREEN(X):return (HValue(X[3])*16+HValue(X[4])) & 0xFF
def BLUE(X) :return (HValue(X[5])*16+HValue(X[6])) & 0xFF
#■ ＤＩＢファイルヘッダ
class DIBfileHeader:
    def __init__(self):
        self.Type=0x4D42
    #■ヘッダ読み込み
    def read(self, fp):
        fp.seek(0)
        s=fp.read(2); self.Type=little16(s)
        s=fp.read(4); self.Size=little32(s)
        s=fp.read(4); s=fp.read(4)
        self.OffsetBits=little32(s)
    #■ヘッダ表示
    def print(self):
        print("            ファイル種類： ",hex(self.Type),"(",self.Type,")")
        print("    ファイル全体のサイズ： ",hex(self.Size),"(",self.Size,")")
        print("ピクセルビットオフセット： ",hex(self.OffsetBits),"(",self.OffsetBits,")")
    #■ヘッダ書き込み
    def write(self, fp):
        fp.seek(0)
        s=fp.write(bAr16(self.Type))
        s=fp.write(bAr32(self.Size))
        s=fp.write(bAr32(0))
        s=fp.write(bAr32(self.OffsetBits))
#■ Bitmap情報
class BitmapInfo:
    def __init__(self):
        self.bcSize=0
    #s■ビットマップ情報読み込み
    def read(self,fp):
        fp.seek(14)      # 14byte 目に移動
        s=fp.read(4)
        self.Size=little32(s)
        self.RGBsize=4
        if self.Size==12: # IBM BMP(BitmapInfoのサイズが12のときIBM形式)
            self.RGBsize=3
            s=fp.read(2); self.width         =little16(s)
            s=fp.read(2); self.height        =little16(s)
            s=fp.read(2); self.Planes        =1
            s=fp.read(2); self.BitCount      =little16(s)
        else:            # Microaoft Bmp
            s=fp.read(4); self.width         =little32(s)
            s=fp.read(4); self.height        =little32(s)
            s=fp.read(2); self.Planes        =1
            s=fp.read(2); self.BitCount      =little16(s)
            s=fp.read(4); self.Compression   =little32(s)
            s=fp.read(4); self.SizeImage     =little32(s)
            s=fp.read(4); self.XPperMeter    =little32(s)
            s=fp.read(4); self.YPperMeter    =little32(s)
            s=fp.read(4); self.ColorUsed     =little32(s)
            s=fp.read(4); self.ColorImportant=little32(s)

    #■Bitmap情報の表示
    def print(self):
        print("                    Size : ", hex(self.Size          ),"(",self.Size,")")
        print("                   width : ", hex(self.width         ),"(",self.width,")")
        print("                  height : ", hex(self.height        ),"(",self.height,")")
        print("               Bit Count : ", hex(self.BitCount      ),"(",self.BitCount,")")
        print("             Compression : ", hex(self.Compression   ),"(",self.Compression,")")
        print("             Image Bytes : ", hex(self.SizeImage     ),"(",self.SizeImage,")")
        print("           X pixel/Meter : ", hex(self.XPperMeter    ),"(",self.XPperMeter,")")
        print("           Y pixel/Meter : ", hex(self.YPperMeter    ),"(",self.YPperMeter,")")
        print("   Number of Color Table : ", hex(self.ColorUsed     ),"(",self.ColorUsed,")")
        print("         Important Color : ", hex(self.ColorImportant),"(",self.ColorImportant,")")
    def write(self,fp):
        fp.seek(14)      # 14byte 目に移動
        s=fp.write(bAr32(self.Size))
        if self.Size==12: # IBM BMP(BitmapInfoのサイズが12のときIBM形式)
            s=fp.write(bAr16(self.width)); s=fp.write(bAr16(self.height))
            s=fp.write(bAr16(self.Planes)); s=fp.write(bAr16(self.BitCount))
        else:            # Microaoft Bmp
            s=fp.write(bAr32(self.width)); s=fp.write(bAr32(self.height))
            s=fp.write(bAr16(self.Planes)); s=fp.write(bAr16(self.BitCount))
            s=fp.write(bAr32(self.Compression))
            s=fp.write(bAr32(self.SizeImage))
            s=fp.write(bAr32(self.XPperMeter))
            s=fp.write(bAr32(self.YPperMeter))
            s=fp.write(bAr32(self.ColorUsed))
            s=fp.write(bAr32(self.ColorImportant))

                             
#■ Bitmapクラス(カラーマップとビットマップの元の情報を含む）
class Bitmap:
    def __init__(self, header,info):
        self.header=header
        self.info=info
    #■カラーマップとビットマップの元の情報を読み込む
    def read(self,fp):
        if self.info.BitCount>=24:
            self.ColorMapLen=0
            self.ColorMap=[]
        else:
            self.ColorBufLen=self.info.RGBsize*(1<<self.info.BitCount)
            self.ColorBuf=fp.read(self.ColorBufLen)
            self.createColorMap()
        self.Len=self.header.Size-self.header.OffsetBits
        self.Buffer=safe_read(fp,self.Len)
    #■ファイル内のカラーマップをPythonのカラー形式に変換する
    def createColorMap(self):
        RGBsize=self.info.RGBsize
        N=1<<self.info.BitCount
        self.ColorMapLen=N
        self.ColorMap=[0 for j in range(N)]
        k=0
        pClr=self.ColorBuf
        for i in range(N):
            self.ColorMap[i]=RGB(pClr[k+2],pClr[k+1],pClr[k])
            k+=RGBsize
            
#■ Canvas表示クラス
class CBitmap:
    def __init__(self, canvas, width,height,BackgroundColor='#FFFFFF'):
        self.Canvas=canvas
        self.width=width
        self.height=height
        #キャンバス内を擬似的なピクセル(1×1の正方形)で埋め尽くす
        self.clearAll()
        self.ID=array2(width,height)
        self.colorNo=array2(width,height)
        self.BackgroundColor=BackgroundColor
        for Y in range(height):
            for X in range(width):
                self.ID[X][Y]=canvas.create_rectangle(X,Y,X,Y,fill=BackgroundColor, outline='')
    #■キャンバス内itemをすべてクリア
    def clearAll(self): self.Canvas.delete(ALL)
    #■24ビットＢＭＰの表示
    def draw24Bitmap(self, bitmap):
        w=bitmap.info.width; h=bitmap.info.height
        if h<0: # 負のとき画面上位から下方向へ
            Y=0; dY=1; yLoop=-h
        else: # 正のとき画面下位から上方向へ
            Y=h-1; dY=-1;yLoop=h
        bMap=bitmap.Buffer
        #24bitsのとき3バイト単位，32ビットのとき4バイト単位
        NCount=3
        if bitmap.info.BitCount>24: NCount=4
        k=0
        for j in range(yLoop):
            if (Y>=0) and (Y<=self.width):
                k=int((k+3)/4)*4  # 4バイト境界に合わせる(32ビットのとき変化なし)
                for X in range(w):
                    if X<=self.width:
                        self.Canvas.itemconfig(self.ID[X][Y],
                                               fill="#%02X%02X%02X" % (bMap[k+2],bMap[k+1],bMap[k]))
                    k+=NCount; X+=1
            Y+=dY
    #■24ビットＢＭＰの設定
    def set24Bitmap(self, bitmap):
        w=bitmap.info.width; h=bitmap.info.height
        if h<0: # 負のとき画面上位から下方向へ
            Y=0; dY=1; yLoop=-h
        else: # 正のとき画面下位から上方向へ
            Y=h-1; dY=-1; yLoop=h
        bMap=bitmap.Buffer
        #24bitsのとき3バイト単位，32ビットのとき4バイト単位
        NCount=3
        if bitmap.info.BitCount>24: NCount=4
        k=0; buf=bytearray(b'')
        for j in range(yLoop):
            for X in range(w):
                if  (Y>=0) and (Y<=self.width) and X<=self.width:
                    CL=self.Canvas.itemcget(self.ID[X][Y],option='fill')
                    buf.append(BLUE(CL))
                    buf.append(GREEN(CL))
                    buf.append(RED(CL))
                else:
                    CL= self.BackgroundColor
                    buf.append(BLUE(CL))
                    buf.append(GREEN(CL))
                    buf.append(RED(CL))
                if NCount==4: buf.append(0)
                k+=NCount; X+=1
            kk=int((k+3)/4)*4  # 行の最後を4バイト境界直前に合わせる
            while k<kk: buf.append(0); k+=1
            Y+=dY
        return buf
        
    #■8ビットＢＭＰの表示
    def draw8Bitmap(self, bitmap):
        w=bitmap.info.width; h=bitmap.info.height
        if h<0: # 負のとき画面上位から下方向へ
            Y=0; dY=1; yLoop=-h
        else: # 正のとき画面下位から上方向へ
            Y=h-1;dY=-1; yLoop=h
        bMap=bitmap.Buffer; NCount=1  # 8ビットのとき単純に1バイト単位
        k=0
        for j in range(yLoop):
            if (Y>=0) and (Y<=self.width):
                k=int((k+3)/4)*4 # 4バイト境界に合わせる(32bits / 8bits=4)
                for X in range(w):
                    if X<=self.width:
                        CL=bitmap.ColorMap[bMap[k]]
                        self.Canvas.itemconfig(self.ID[X][Y], fill=CL)
                        self.colorNo[X][Y]=bMap[k]
                    k+=NCount; X+=1
            Y+=dY
    #■色間の距離
    def distanceColor(self, color, R, G, B):
        RR = color[0]-R; GG = color[1]-G; BB = color[2]-B;
        return abs(RR)+abs(GG)+abs(BB)#高速化のためこれで近似
    #■最も近いカラーテーブル探索
    def nearColorID(self, colormap, color):
        R=RED(color); G=GREEN(color); B=BLUE(color)
        minD=self.distanceColor(colormap[0],R,G,B)
        if minD==0: return 0
        minI=0
        for i in range(1,len(colormap)):
            D=self.distanceColor(colormap[i],R,G,B)
            if D== 0: return i
            if D<minD: minI=i; minD=D
        return minI
    #■検索用カラーテーブル設定
    def setRetColor(self, bitmap):
        Cmap=[]
        for i in range(len(bitmap.ColorMap)):
            Cmap.append([RED(bitmap.ColorMap[i]),GREEN(bitmap.ColorMap[i]),BLUE(bitmap.ColorMap[i])])
        return Cmap        
    #■8ビットＢＭＰの設定
    def setRCtab(self,bitmap):
        Rtab=[[] for j in range(256)]
        for i in range(bitmap.ColorMapLen):
            R=RED(bitmap.ColorMap[i]); G=GREEN(bitmap.ColorMap[i])
            B=BLUE(bitmap.ColorMap[i])
            ID=int((R+G+B)/3); Rtab[ID].append([R,G,B,i])
        ist=0; ied=1
        while(ist<255):
            while(Rtab[ied]==[]): ied+=1
            if(ied-ist)>1:
                imid=int((ist+ied)/2)
                for i in range(ist,imid): Rtab[i]=Rtab[ist]
                for i in range(imid,ied): Rtab[i]=Rtab[ied]
            ist=ied; ied=ist+1
        return Rtab
    def nearColorID2(self, colormap, color):
        R=RED(color);G=GREEN(color);B=BLUE(color)
        ID=int((R+G+B)/3)
        minD=self.distanceColor(colormap[ID][0],R,G,B)
        if minD==0: return colormap[ID][0][3]
        minI=0
        for i in range(1,len(colormap[ID])):
            D=self.distanceColor(colormap[ID][i],R,G,B)
            if D== 0: return colormap[ID][i][3]
            if D<minD: minI=i; minD=D
        return colormap[ID][minI][3]
    def set8Bitmap(self, bitmap):
        Cmap=self.setRCtab(bitmap) #検索用カラーテーブル設定
        w=bitmap.info.width; h=bitmap.info.height
        if h<0: # 負のとき画面上位から下方向へ
            Y=0; dY=1; yLoop=-h
        else: # 正のとき画面下位から上方向へ
            Y=h-1; dY=-1;yLoop=h
        bMap=bitmap.Buffer;NCount=1  # 8ビットのとき単純に1バイト単位
        k=0; buf=bytearray(b'')
        BackColorID=self.nearColorID2(Cmap, self.BackgroundColor)
        for j in range(yLoop):
            for X in range(w):
                if X<=self.width and (Y>=0) and (Y<=self.width):
                    CL=self.Canvas.itemcget(self.ID[X][Y],option='fill')
                    if CL==bitmap.ColorMap[self.colorNo[X][Y]]:
                        buf.append(self.colorNo[X][Y])
                    else:buf.append(self.nearColorID2(Cmap, CL))
                else: buf.append(BackColorID)
                k+=NCount;X+=1
            kk=int((k+3)/4)*4 # 4バイト境界に合わせる(32bits / 8bits=4)
            while k<kk:buf.append(0); k+=1
            Y+=dY
        return buf
    #■4ビットＢＭＰの表示
    def draw4Bitmap(self, bitmap):
        w=bitmap.info.width; h=bitmap.info.height
        if h<0: # 負のとき画面上位から下方向へ
            Y=0; dY=1; yLoop=-h
        else: # 正のとき画面下位から上方向へ
            Y=h-1; dY=-1;yLoop=h
        bMap=bitmap.Buffer;NCount=1; k=0
        for j in range(yLoop):
            if (Y>=0) and (Y<=self.width):
                k=int((k+7)/8)*8 # 4バイト境界に合わせる(32bits / 4bits = 8) 
                for X in range(w):
                    if X<=self.width:
                        kk=int(k/2)  
                        if (k & 1): # 奇数番目のとき下位4bits
                            self.colorNo[X][Y]=bMap[kk] & 0xF
                            CL=bitmap.ColorMap[self.colorNo[X][Y]]
                        else:       # 偶数番目のとき上位4bits
                            self.colorNo[X][Y]=bMap[kk] >> 4
                            CL=bitmap.ColorMap[self.colorNo[X][Y]]
                        self.Canvas.itemconfig(self.ID[X][Y], fill=CL)
                    k+=NCount; X+=1
            Y+=dY
    #■4ビットＢＭＰの設定
    def set4Bitmap(self, bitmap):
        Cmap=self.setRCtab(bitmap) #検索用カラーテーブル設定
        w=bitmap.info.width;h=bitmap.info.height
        if h<0: # 負のとき画面上位から下方向へ
            Y=0; dY=1; yLoop=-h
        else: # 正のとき画面下位から上方向へ
            Y=h-1; dY=-1; yLoop=h
        bMap=bitmap.Buffer; buf=bytearray(b'')
        NCount=1; k=0
        BackColorID=self.nearColorID2(Cmap, self.BackgroundColor)
        for j in range(yLoop):
            beforC=0
            for X in range(w):
                kk=int(k/2)  #値の格納位置
                if X<=self.width and (Y>=0) and (Y<=self.width):
                    if (k & 1): # 奇数番目のとき前のカラーに下位4bits付加設定
                        CL=self.Canvas.itemcget(self.ID[X][Y],option='fill')
                        buf.append(beforC | self.nearColorID2(Cmap, CL))
                    else:       # 偶数番目のとき値を上位4bitsに設定しておく
                        CL=self.Canvas.itemcget(self.ID[X][Y],option='fill')
                        beforC=(self.nearColorID2(Cmap, CL)<<4) & 0xFF
                else:
                    if (k & 1): # 奇数番目のとき前のカラーに下位4bits付加
                        buf.append(beforC | BackColorID)
                    else:       # 偶数番目のとき値を上位4bitsに設定しておく
                        beforC=(BackColorID<<4) & 0xFF
                k+=NCount; X+=1
            if (k & 1): buf.append(beforC | 0); k+=1 # 奇数番目のとき
            kk=int((k+7)/8)*8 # 4バイト境界に合わせる(32bits / 4bits=8)
            while k<kk: buf.append(0); k+=2
            Y+=dY
        return buf
    #■1ビットＢＭＰの表示
    def draw1Bitmap(self, bitmap):
        w=bitmap.info.width; h=bitmap.info.height
        if h<0: # 負のとき画面上位から下方向へ
            Y=0; dY=1; yLoop=-h
        else: # 正のとき画面下位から上方向へ
            Y=h-1; dY=-1; yLoop=h
        bMap=bitmap.Buffer; NCount=1; k=0
        for j in range(yLoop):
            if (Y>=0) and (Y<=self.width):
                k=int((k+31)/32)*32  # 32bits境界に合わせる
                for X in range(w):
                    if X<=self.width:
                        kk=int(k/8)  # kk: バイト単位の位置
                        imap = (bMap[kk]>>(7 - (k %8))) & 1 # 0または1
                        CL=bitmap.ColorMap[imap]
                        self.Canvas.itemconfig(self.ID[X][Y], fill=CL)
                    k+=NCount;X+=1
            Y+=dY
    #■1ビットＢＭＰの設定
    def set1Bitmap(self, bitmap):
        Cmap=self.setRetColor(bitmap) #検索用カラーテーブル設定
        w=bitmap.info.width; h=bitmap.info.height
        if h<0: # 負のとき画面上位から下方向へ
            Y=0; dY=1; yLoop=-h
        else: # 正のとき画面下位から上方向へ
            Y=h-1; dY=-1; yLoop=h
        bMap=bitmap.Buffer; buf=bytearray(b'')
        NCount=1;k=0
        for j in range(yLoop):
            beforC=0
            for X in range(w):# kk: バイト単位の位置
                kk=int(k/8);imod = k % 8
                if X<=self.width and (Y>=0) and (Y<=self.width):
                    CL=self.Canvas.itemcget(self.ID[X][Y],option='fill')
                    beforC=(beforC<<1) | self.nearColorID(Cmap, CL)
                    if imod == 7: buf.append(beforC); beforC=0
                else:
                    CL=self.BackgroundColor
                    beforC=(beforC<<1) | self.nearColorID(Cmap, CL)
                    if imod == 7: buf.append(beforC); beforC=0
                k+=NCount; X+=1
            imod = k % 8
            if imod !=0:  # ビット出力残りの処理
                beforC = beforC << (7-imod); buf.append(beforC)
                k+=(8-imod)
            kk=int((k+31)/32)*32 # 32バイト境界に合わせる(32bits /1bits=32)
            while k<kk: buf.append(0); k+=8
            Y+=dY
        return buf    
    #■ＢＭＰの表示
    def drawBitmap(self, bitmap):
        if bitmap.info.BitCount>=24:
            self.draw24Bitmap(bitmap)
        elif bitmap.info.BitCount==8:
            self.draw8Bitmap(bitmap)
        elif bitmap.info.BitCount==4:
            self.draw4Bitmap(bitmap)
        elif bitmap.info.BitCount==1:
            self.draw1Bitmap(bitmap)       
        else:
            if bitmap.info.BitCount<24:
                dsplayColorMap(self.canvas,bitmap)
            messagebox.showinfo("BmpDisplay", str(bitmap.info.BitCount)+"ビットの画像は今だサポートしていません。")
    
    #■ピクセル取り出し(返却値はPythonのカラー形式（string型))
    def GetPixel(self, X,Y):
        if X<0 or X>=self.width or Y<0 or Y>=self.height:
            return '#000000'
        return self.Canvas.itemcget(self.ID[X][Y], option='fill')
    def GetPixelList(self, X,Y):
        if X<0 or X>=self.width or Y<0 or Y>=self.height:
            return [0,0,0]
        CL=self.Canvas.itemcget(self.ID[X][Y], option='fill')
        return [RED(CL),GREEN(CL),BLUE(CL)]
    def SetPixel(self,X,Y, CL):
        if X<0 or X>=self.width or Y<0 or Y>=self.height:
            return
        self.Canvas.itemconfig(self.ID[X][Y], fill=CL)
    def SetPixelList(self,X,Y, CLL):
        if X<0 or X>=self.width or Y<0 or Y>=self.height:
            return
        self.Canvas.itemconfig(self.ID[X][Y], fill=RGB(CLL[0],CLL[1],CLL[2]))
       
#■ カラーマップ表示(Python用形式に変換したので，前バージョンを変更)    
def displayColormap(canvas,bitmap):
    k=0
    Y=-20
    for i in range(bitmap.ColorMapLen):
        if i% 16==0:
            Y+=30
            X=10
        CL=bitmap.ColorMap[i]
        canvas.create_rectangle(X,Y,X+20,Y+20, fill=CL,outline='black')
        X+=30
#■ キャンバスにビットマップを表示    
def drawBitmap(canvas, bitmap):
    cbit=CBitmap(canvas, bitmap.info.width,bitmap.info.height)
    cbit.drawBitmap(bitmap)
    return cbit
#■ ビットマップ読込み    
def readBitmap(filename):
    try:
        fp=open(filename,'rb')
        fp.seek(0)
        #ファイル・ヘッダ読込み
        header=DIBfileHeader()
        header.read(fp)
        header.print()
        if header.Type!=0x4D42:
            messagebox.showerror("ファイル種類", "BMPファイルではありません。")
            fp.close()
            return []
        else:
            #ビットマップ情報読込み
            info=BitmapInfo()
            info.read(fp)
            info.print()
            #カラーテーブルおよびビットマップ読み込み
            bitmap=Bitmap(header,info)
            bitmap.read(fp)
            fp.close()
            return bitmap
    except IOError:
        raise IOError("Unspported BMP header type)" )
        return []
#■ ビットマップ出力
def writeBitmap(filename, cbit,bitmap):
    try:
        fp=open(filename,'wb')
        fp.seek(0)
        #ファイル・ヘッダおよびビットマップ情報出力
        bitmap.header.write(fp)
        bitmap.info.write(fp)
        #カラーマップおよびビットマップ出力
        if bitmap.info.BitCount>=24:
            fp.write(cbit.set24Bitmap(bitmap))
        elif bitmap.info.BitCount==8:
            fp.write(bitmap.ColorBuf)
            fp.write(cbit.set8Bitmap(bitmap))
        elif bitmap.info.BitCount==4:
            fp.write(bitmap.ColorBuf)
            fp.write(cbit.set4Bitmap(bitmap))
        elif bitmap.info.BitCount==1:
            fp.write(bitmap.ColorBuf)
            fp.write(cbit.set1Bitmap(bitmap))
        else:
            messagebox.showerror("ファイル種類",
                                 "このビット長はサポートされていません。")
            fp.close()
            return False    
        fp.close()
        return True
    except IOError:
        raise IOError("Unspported BMP header type)" )
        return False
#
#■ CreateBitmapクラス：新たに画像を描く際，このクラスを使う。
#       ただし，24ビット形式のみを用意しています。
#
class CreateBitmap:
    def __init__(self,width,height):
        bitPixel=24; headerSize=14;infoSize=40
        nImage=int(((width*bitPixel)+31)/8)*height

        self.header= DIBfileHeader()
        self.header.Size=headerSize+infoSize+nImage
        self.header.OffsetBits=40

        self.info=BitmapInfo()
        self.info.Size=40
        self.info.width=width
        self.info.height=height
        self.info.Planes=1
        self.info.BitCount=bitPixel
        self.info.Compression=0
        self.info.SizeImage=width*height*3
        self.info.XPperMeter=0
        self.info.YPperMeter=0
        self.info.ColorUsed=0
        self.info.ColorImportant=0

        self.bitmap=Bitmap(self.header, self.info)
        self.bitmap.ColormapLen=0
        self.bitmap.Colormap=[]
        self.bitmap.Len=self.bitmap.header.Size-self.bitmap.header.OffsetBits
        self.bitmap.Buffer=[0xFF for j in range(self.bitmap.Len) ]
#
#■カラーコードの取り出し
#  本プログラムでは色をカラーコードで指定することを前提にしています。
#  どうしてもカラー名で指定したい人のために,
#  よく使われそうな名前だけをピックアップしてこの関数を用意しました。
#
colorNames=[['red'     ,'FF0000'],['yellow'    ,'FFFF00'],['lime'   ,'00FF00'],
            ['cyan'    ,'00FFFF'],['blue'      ,'0000FF'],['magenta','FF00FF'],
            ['white'   ,'FFFFFF'],['lightgray' ,'D3D3D3'],['silver' ,'C0C0C0'],
            ['darkgray','A9A9A9'],['gray'      ,'808080'],['black'  ,'000000'],
            ['darkblue','000088'],['darksyan'  ,'008888'],['green'  ,'008000'],
            ['orange'  ,'FFA500'],['purple'    ,'800080'],['pink'   ,'FFC0CB'],
            ['violet'  ,'EE82EE'],['blueviolet','8A2BE2'],['ivory'  ,'FFFFF0']]
def colorCode(C):
    CC=C.lower()
    for i in range(len(colorNames)):
        if CC==colorNames[i][0]: return '#'+colorNames[i][1]

#■ RGBリストを得る
def getRGBList(cbit):
    X=[]
    for x in range(len(cbit.ID)):
        Y=[]
        for y in range(len(cbit.ID[x])):
            CL=cbit.Canvas.itemcget(cbit.ID[x][y],option='fill')
            Y.append([RED(CL),GREEN(CL),BLUE(CL)])
        X.append(Y)
    return X
